function [nvar, names, types, header_offsets, freq] = nexf_info(fid, verbose)
% Read general file information from a .nex file
%
% [nvar, names, types, header_offsets, freq] = nex_info(fid, verbose)
%
% INPUT:
%   fid - fileID
%   verbose - display translation info
% OUTPUT:
%   nvar - number of variables in the file
%   names - [nvar 64] array of variable names
%   types - [1 nvar] array of variable types
%           Interpretation of type values: 
%           0-neuron, 1-event, 2-interval, 3-waveform, 
%           4-population vector, 5-continuous variable, 6 - marker
%   header_offsets - offset from bof of each variable's header in the file
%   freq - frequency of collection of timestamped data

names = zeros(1, 64);

frewind(fid);

%read file info
magic = fread(fid, 1, 'int32');
version = fread(fid, 1, 'int32');
comment = fread(fid, 256, 'char');
freq = fread(fid, 1, 'double');
tbeg = fread(fid, 1, 'int32');
tend = fread(fid, 1, 'int32');
nvar = fread(fid, 1, 'int32');
fseek(fid, 260, 'cof');

%read variable info
for i=1:nvar
	header_offsets(i) = ftell(fid);
    types(i) = fread(fid, 1, 'int32');
	var_version(i) = fread(fid, 1, 'int32');
	names(i,:) = fread(fid, [1 64], 'char');
%	data_offsets(i) = fread(fid, 1, 'int32');
    fseek(fid, 136,'cof');
end
names = upper(setstr(names));

if(verbose)
    disp(strcat('frequency        = ', num2str(freq)));
    disp(strcat('duration (sec)   = ', num2str((tend - tbeg)/freq)));
    disp(strcat('# of variables   = ', num2str(nvar)));
    disp(strcat('# of markers     = ', num2str(length(find(types==6)))));
    disp(strcat('# of spikes      = ', num2str(length(find(types==0)))));
    disp(strcat('# of waveforms   = ', num2str(length(find(types==3)))));
    disp(strcat('# of events      = ', num2str(length(find(types==1)))));
    disp(strcat('# of cont. vars. = ', num2str(length(find(types==5)))));    
    disp('The names of the variables are:');
    disp(names);
end