function [posX, posY, down, up, yestrials] = touch (c, a, b, m)

%@CORTEX/TOUCH   returns touchscreen positions from a @cortex object
%
%   [POSX,POSY,DOWN,UP,TRIALS] = TOUCH(COBJ,KEY,MODE)  retrieves XY
%   positions and timing of touchscreen data from the trials selected
%   by a single @key object's KEY, from the COBJ @cortex object. The
%   optional string argument MODE determines how events are analyzed.
%   CORTEX samples the touchscreen state at the rate specified in
%   cortex.cfg, and data are collected any time a contact is detected.
%   Thus, since a typical "touch response" may last longer than the
%   sampling interval, multiple events may be recorded for each
%   discrete motor response. Valid MODEs are:
%
%   'all':   all samples are included in the output (this is the
%            default, used if MODE is omitted)
%   'down':  only the XY position of the first contact with the
%            screen is returned
%   'up':    only the XY position just prior to the release of a
%            contact are returned
%   'mean':  XY positions are averaged over the entire duration of
%            the contact
%
%   POSX and POSY are cell arrays of X and Y touchscreen coordinates;
%   DOWN and UP are cell arrays with the timing in ms of the initial 
%   contact and of the release of each touch response, relative to the
%   cue event specified by the KEY; if MODE=='all' then both DOWN and
%   UP contain the timing of each touchscreen sample. Each  cell of
%   these return arrays contains an array of values for a given trial.
%   TRIALS is an array of trial numbers selected by the key.
%
%   POSMATRIX = TOUCH(...)  same as above, except that X & Y positions,
%   down and up timing, and trial number from which the values are
%   retrieved are stored in the 5-column double matrix POSMATRIX.
%
%   [...] = TOUCH(COBJ,TSET,CUE,MODE)  the key can be replaced by
%   TSET, a numeric vector of trials to analyze and CUE, a scalar
%   specifying the time-locking encode.
%
%   [...] = TOUCH(COBJ,CUE,MODE)  if neither KEY nor TSET are used,
%   then all trials in COBJ are included.
%
%   [...] = TOUCH(COBJ,MODE)  if CUE is omitted, then timings are
%   computed relative to the beginning of each trial.
%
% See also: @CORTEX/TRIALTOUCH
%
% Last modified: 11 Oct 04

% created on 24 Jun 04, based on eogpos.m

if isempty (c.tch), error ('no touchscreen data in this file'); end

cue = [];
mode = 0;
switch nargin
   case 1
      % (COBJ)
      yestrials = 1:c.trials;
   case 2
      if isa (a, 'rules') | isa (a, 'key')
         % (COBJ,KEY)
         yestrial = trialset (c, a);
         cue = a.cue;
         if length(cue)>1
            warning ('multiple cue events in key: only the first will be considered');
            cue = cue(1);
         end
      elseif isnumscalar(a)
         % (COBJ,CUE)
         cue = a;
         yestrials = 1:c.trials;
      else
         % (COBJ,MODE)
         mode = whatmode (a);
         yestrials = 1:c.trials;
      end
   case 3
      if isnumvector(a) & isnumscalar(b)
         % (COBJ,TSET,CUE)
         yestrials = a;
         cue = b;
      elseif isnumscalar(a) & isstring(b)
         % (COBJ,CUE,MODE)
         yestrials = 1:c.trials;
         cue = a;
         mode = whatmode(m);
      elseif (isa (a, 'rules') | isa (a, 'key')) & isstring(b)
         % (COBJ,KEY,MODE)
         yestrials = trialset (c, a);
         cue = a.cue;
         mode = whatmode(b);
      else
         error ('bad argument list');
      end
   case 4
      % (COBJ,TSET,CUE,MODE)
      if isnumvector(a) & isnumscalar(b) & isstring(m)
         yestrials = a;
         cue = b;
         mode = whatmode(m);
      else
         error ('bad argument list');
      end
   otherwise
      error ('wrong number of arguments')
end

posmatrix = [];
cueless = [];
ct = 0;
for trial=yestrials
   [X,Y,D,U] = gettouch(c,trial,cue,mode);
   if any(isnan(D)), cueless = [cueless, trial]; end
   if nargout == 1
      posmatrix = [posmatrix; [X Y D U repmat(trial,length(X),1)]];
   else
      ct=ct+1;
      posX {ct} = X;
      posY {ct} = Y;
      down {ct} = D;
      up   {ct} = U;
   end
end

% % if nargout == 1,
% %    posmatrix = zeros (c.tch.ptr(end)-1, 5);
% % else
% %    ct=0;
% % end
% % 
% % for trial=yestrials
% %    range = c.tch.ptr (trial) : c.tch.ptr (trial+1)-1;
% %    X = c.tch.X (range);
% %    Y = c.tch.Y (range);
% %    T = double (c.tch.T (range));
% %    
% %    if mode
% %       changes = find (diff(T) > c.tch.rate/2*3);
% %       did = [1; changes+1];
% %       uid = [changes; length(T)];
% %       D = T(did);
% %       U = T(uid);
% %       switch mode
% %          case 1 % down
% %             X = X(did);
% %             Y = Y(did);
% %          case 2 % up
% %             X = X(uid);
% %             Y = Y(uid);
% %          case 3 % mean
% %             for st = 1:length(D)
% %                subr = D(st):U(st);
% %                X(st) = mean(X(subr));
% %                Y(st) = mean(Y(subr));
% %             end
% %       end
% %    else
% %       D = T;
% %       U = T;
% %    end
% %    
% %    % if a cue is given, subtract cue onset time from c.tch.T
% %    if ~isempty (cue)
% %       fi = find (c.codes(trial,:) == cue);
% %       if isempty (fi)
% %          cueless = [cueless trial];
% %          T(:) = NaN;
% %       else
% %          T = T - c.times(trial, fi);
% %       end
% %    end
% %    
% %    % set up return values
% %    if nargout == 1
% %       posmatrix = [posmatrix; [X Y D U repmat(trial,length(X),1)]];
% %    else
% %       ct=ct+1;
% %       posX {ct} = X;
% %       posY {ct} = Y;
% %       down {ct} = D;
% %       up   {ct} = U;
% %    end
% % end

if nargout == 1, posX = posmatrix; end

if ~isempty (cueless)
   warning ('no cue events found in the following trial(s):');
   disp (cueless);
end