function varargout = anovan (sp, start, bin, factors, varargin)

%SPIX/ANOVAN  n-way analysis of variance on @spix data
%
%   [P,T,STATS,TERMS] = ANOVAN(SP,START,BIN,FACTORS) performs n-way
%   anovas on all units currently included in SP, on a BIN-ms-long
%   epoch beginning START ms from the cue event. FACTORS specifies one
%   or more of the previously added factors; it can be a numeric
%   vector, a string, or a cell array of strings containing the factor
%   labels. P is a matrix of p-values (one column per unit); T is a
%   cell array of anova tables (one table per unit); STATS is an
%   structure array of statistics (one structure per unit); TERMS is a
%   matrix of terms used in the analysis (one column per unit). See
%   Matlab's ANOVAN help text for more information on the output
%   arguments.
%
%   [...] = ANOVAN(...,DETAILS) performs ANOVAs only on the units
%   specified by the DETAILS argument pair; only one pair can be used
%   (either 'units' or 'subsets').
%
%   The following additional arguments can be specified in order to
%   fine-tune the analysis:
%   [...] = ANOVAN(...,'MODEL')
%   [...] = ANOVAN(...,'MODEL',SSTYPE)
%   [...] = ANOVAN(...,'MODEL',SSTYPE,'DISPLAYOPT')
%
%   See Matlab's ANOVAN help text for an explanation of these arguments.
%   Note that unlike Matlab's ANOVAN, factors are automatically labeled
%   (i.e., the GNAME argument is not available here) and DISPLAYOPT is
%   'off' by default, in order to avoid the clutter caused by performing
%   the analyses on large numbers of units at once.
%
%Last modified: 31 Aug 03

error (nargchk (4, 10, nargin));
argoffset = 4;
sset = sp.inclunits;
if nargin > 4 & ischar (varargin{1})
   sss = minifield (varargin{1});
   if strcmp (sss, 'uni')
      sset = parsegrouping (sp.info, varargin{2});
      argoffset = 6;
   elseif strcmp (sss, 'sub')
      subs = parsegrouping (sp.subsets, varargin{2});
      if ~isnumscalar (subs), error ('only one subset, please'); end
      sset = sp.subsets(subs).units;
      argoffset = 6;
   end
end

%%% move to trialfiringrate!!!
%%%frates = trialfiringrate (sp, start, bin, 

% calculating firing rates for ALL trials in spix
chunk = sp.spix (:, start-sp.offset:start+bin-sp.offset-1);
frates =  full(sum (chunk, 2) / size(chunk,2) * 1000);
disp (['firing rates computed on ' num2str(length(frates)) ' trials']);

% preparing the level labels
facts = parsegrouping (sp.factors, factors)
okeys = [];
for ke = 1:sp.k.len
   if all (sp.factable(facts, ke))
      okeys = [okeys ke];
      for fa = 1:length(facts)
         thisf = facts(fa);
         gr{ke,fa} = sp.factors(thisf).levels(sp.factable(thisf,ke)).label;
         lab{fa} = sp.factors(thisf).label;
      end
   end
end
disp ('the following keys were selected:')
disp (okeys)

% the default anovan argument list 
arglist = {'linear' 3 lab 'off'};
% modify the list if matching arguments were passed
if nargin - argoffset
   arglist{1} = varargin{argoffset-3};
end
if nargin - argoffset > 1
   arglist{2} = varargin{argoffset-2};
end
if nargin - argoffset > 2
   arglist{4} = varargin{argoffset-1};
end

% set up output vars
allp     = [];
alltable = {};
allstats = [];
allterms = [];

% loop for each requested unit
for ce = 1:length(sset)
   % set up anova
   as = annsetup (length(facts));
   yesunit = sp.unit==sset(ce);
   % go through all keys in the spix
   for ke = okeys
      yeskey = sp.key==ke;
      da = frates (yesunit & yeskey);
      as = addata (as, da, gr(ke,:));
   end
   [p, table, stats, terms] = anovan (data(as), group(as), arglist{:});
   allp = [allp p];
   alltable{ce} = table;
   allstats = [allstats stats];
   allterms = [allterms terms];
   disp (sprintf ('anovan performed on unit #%d; %d to go.', sset(ce), length(sset)-ce));
end
   
o{1} = allp;
o{2} = alltable;
o{3} = allstats;
o{4} = allterms;
varargout(1:nargout)=o(1:nargout);