function [dense, uuni, ugrp] = density (sp, win, bin, kwidth, ktype, varargin)

%SPIX/DENSITY  spike-density functions from a @spix object
%
%   DE = DENSITY(SP,WIN,BIN,WIDTH,TYPE) returns
%   SP is a @spix object; WIN is a 2-element vector specifying the start
%   and end ms for the analysis, relative to the cue event; WIDTH is the
%   size in ms of the kernel (larger values produce smoother curves); TYPE
%   is the shape of the kernel and is either 'gauss' or 'boxcar'.
%
%   DE is a 3-D matrix of density values; each row represents one processed
%   unit; each column corresponds to one ms; density plots for different
%   keys or key groupings are arranged along the 3rd dimension.
%
%   If no other arguments are specified, then all currently included units
%   are processed for each of the currently included keys.
%
%   Pairs of 'specs' can be appended to the argument list in order to
%   restrict analysis to certain units and-or keys.
%
%   For example, density(sp,[-100,299],20,'gauss','uni',6:10,'key',1:3)
%   will return a matrix of size (5, 400, 3)
%
%   [DE,UNI,GRP] = DENSITY(...)
%   UNI is a list of units included in the analysis
%   GRP is a list of keys or key groupings analyzed
%
%    SPMAT = SPIKEMATRIX(SP,EPOCH,BIN) returns a matrix of ones (spike
%    occurred) and zeros (no spike) from the  SP @spix object for all
%    currently selected units and for each of the currently selected
%    keys. If EPOCH is a scalar, then the spike matrix covers a BIN-
%    ms-long epoch starting EPOCH ms after the cue event. If EPOCH is
%    a 2-element array, then the 2 values define the start and end
%    points of the epoch (again, in ms relative to the cue event), and
%    BIN (although still required) is ignored.
%
%    EPOCH can also be one of the following strings:
%    'all': selects the entire time window contained in SP
%    'pre': selects the epoch preceding the cue event 
%    'pos': selects the epoch following the cue event
%    'var': instead of time-locking to the cue event, previously
%           defined, cell-by-cell latencies are used to align trials;
%           if 'var' is used, firing rates are computed on a single
%           epoch of BINSIZE duration
%
%    SPMAT is an M-by-N matrix where M is the number of analyzed trial
%    epochs, and N is the epoch length in ms.
%
%    SPMAT = SPIKEMATRIX(...,RESTRICTIONS) returns only trial epochs
%    specified by RESTRICTIONS. See online help for RESTRICTIONS
%    usage.
%
%    [SPMAT,UNITS,CRIT] = SPIKEMATRIX(...) also returns an array of
%    UNITS from which each trial epoch derives, and CRIT, an array of
%    keys, combos, or levels matching each trial.
%Last modified: 23 Aug 03

[spmat, uni, grp] = spikematrix (sp, win, bin, varargin{:});
uuni = unique(uni); nuni = length(uuni);
ugrp = unique(grp); ngrp = length(ugrp);
dense = zeros (nuni, size(spmat,2), ngrp);
for cc = 1:length(uuni)
   for gr = 1:length(ugrp)
      chunk = spmat (uni==uuni(cc) & grp==ugrp(gr), :);
      te = smoothfr (chunk, kwidth, ktype);
      dense(cc,:,gr) = te;
   end
end
