function [frates, uuni, ugrp] = meanfiringrate (sp, win, binsize, varargin)

%@SPIX/MEANFIRINGRATE   Average firing rates from a @spix object
%
%    FRATES = MEANFIRINGRATE(SP,EPOCH,BINSIZE) computes average firing
%    rates from the SP @spix object. If EPOCH is a scalar, then firing
%    rates are computed on a BINSIZE-ms-wide time window starting
%    EPOCH ms after the cue event. If EPOCH is a two-element array,
%    then EPOCH(1) is the first ms and EPOCH(2) is the last ms of the
%    analysis window, which is divided in bins of BINSIZE duration.
%    Separate firing rates, one per bin, are then returned. If the
%    epoch is not a multiple of binsize, trailing milliseconds are
%    ignored.
%
%    EPOCH can also be one of the following strings:
%    'all': selects the entire time window contained in SP
%    'pre': selects the epoch preceding the cue event 
%    'pos': selects the epoch following the cue event
%    'var': instead of time-locking to the cue event, previously
%           defined, cell-by-cell latencies are used to align trials;
%           if 'var' is used, firing rates are computed on a single
%           epoch of BINSIZE duration
%
%    FRATES is an M-by-N-by-P matrix of mean firing rates, where M is
%    the number of units included in the analysis, N is the number of
%    computed bins, and P is the number of keys or combos used in the
%    analysis.
%
%    FRATES = MEANFIRINGRATE(...,RESTRICTIONS) returns only data
%    computed on trial epochs specified by RESTRICTIONS. See online
%    help for RESTRICTIONS usage.
%
%    [FRATES,UNITS,CRIT] = MEANFIRINGRATE(...) also returns UNITS, an
%    array of unit #s from which firing rates were computed, and CRIT,
%    a similar list of keys, combos, or levels that were used in the
%    analysis. It follows that: length(units)==size(frates,1), and
%    length(grps)==size(frates,3);
%
%Last modified: 27 Nov 04

% 23 dec 02: last unnoted changes
% 16 apr 03: help text
% 17 Apr 03: call to truncspm
% 22 Apr 03: sum(spmat, 1) bugfix
% 24 Apr 03: findyestrials replaced by trialset
% 30 Apr 03: bug fix
% 22 May 03: switch from celltrialset to trialset
% 14 Jul 03: fixes
% 13 Aug 03: tiny edits
% 18 Aug 03: call trialfiringrate
% 19 Sep 03: help text revised; frates squeezing removed
% 27 Nov 04: help text revised

error (nargchk (3, 9, nargin));
[tfr, uni, grp] = trialfiringrate (sp, win, binsize, varargin{:});
uuni = unique(uni); nuni = length(uuni);
ugrp = unique(grp); ngrp = length(ugrp);
for cc = 1:nuni
   if sp.verbose
      disp (['computing mean firing rates for unit #' num2str(cc)]);
   end
   for gr = 1:ngrp
      yt = (uni==uuni(cc)) & (grp==ugrp(gr));
      frates (cc, :, gr) = full (mean(tfr(yt,:)));
   end
end

% % % if only one bin was computed, then bring keys/groups to 2nd
% % % dimension
% % if size(frates,2)==1
% %    frates = permute (frates, [1 3 2]);
% % end