function s = ScanCortexFile (source_filename)

% ScanCortexFile.m
%
% S = SCANCORTEXFILE(SOURCE)   returns the structure S containing some
%   elementary information about the SOURCE data file. Its main purpose is
%   to  provide a quick reference to the CORTEX data file structure for the
%   programmer. If the file is somehow corrupted chances are, the function
%   will report an error.
%
% Last modified: 8 June 04

% changes since 28 Jan 99
% 04 may 04: just found out that the EPP buffer *precedes* EOGs!!!
% 08 jun 04: output structure added, including epp channel sorting

UINT8  = 1;
UINT16 = 2;
UINT32 = 4;


f = openfile (source_filename, 'r');

trial = 0;
channels = [];
maxtrials = 2000;
cond_no        = zeros (maxtrials, 1);
repeat_no      = zeros (maxtrials, 1);
block_no       = zeros (maxtrials, 1);
trial_no       = zeros (maxtrials, 1);
eog_rate       = zeros (maxtrials, 1);
KHz_resolution = zeros (maxtrials, 1);
trial_type     = zeros (maxtrials, 1);
response_type  = zeros (maxtrials, 1);
response_error = zeros (maxtrials, 1);

while 1 %not_eof
    
    header_length  = fread (f, 1, 'uint16'); % length of header in bytes; should always be 26
    
    % check for eof and for file integrity (header size should be 26)
    if isempty (header_length)
        disp ('done.');
        break
    elseif header_length ~= 26
        error ('possible file corruption: the header should be 26 bytes long');
    else
        trial = trial + 1;
    end
    
    cond_no       (trial) = fread (f, 1, 'uint16'); % base 0 !!!
    repeat_no     (trial) = fread (f, 1, 'uint16'); % base 0 !!! 
    block_no      (trial) = fread (f, 1, 'uint16'); % base 0 !!!
    trial_no      (trial) = fread (f, 1, 'uint16'); % base 1 !!!
    
    timebuf_size = fread (f, 1, 'uint16'); % length in bytes of the time stamp buffer
    codebuf_size = fread (f, 1, 'uint16'); % length in bytes of the encodes buffer
    eogbuf_size  = fread (f, 1, 'uint16'); % length in bytes of the eog buffer
    eppbuf_size  = fread (f, 1, 'uint16'); % length in bytes of the epp buffer
    
    eog_rate      (trial) = fread (f, 1, 'uint8' ); % ms between eog samples. set in General Parameters
    KHz_resolution(trial) = fread (f, 1, 'uint8' ); % data collection sampling rate. idem
    
    trial_type    (trial) = fread (f, 1, 'uint16'); % an attribute of the condition
    response_type (trial) = fread (f, 1, 'uint16'); % the argument passed to the response_xxx() function
    response_error(trial) = fread (f, 1, 'uint16'); % the value returned by the response_xxx() function
    
    % a check for file integrity
    if timebuf_size/UINT32 ~= codebuf_size/UINT16
        warning ('possible file corruption: timebuf vs. codebuf mismatch');
    end
    
    % buffers
    times          = fread (f, timebuf_size/UINT32, 'uint32');
    codes          = fread (f, codebuf_size/UINT16, 'uint16');
    % WARNING: epps values are really 'signed'; uint16s are used here to
    % allow bitand to read the 4 least significan bits carrying the channel
    % number (usually 1 & 2 for analog data acquired via the I/O board, and
    % 14 & 15 for touchscreen data coming from the serial port.
    epp            = fread (f, eppbuf_size /UINT16, 'uint16');
    eog            = fread (f, eogbuf_size /UINT16, 'int16');
    if eppbuf_size
       ch = bitand (epp, 15);
       channels = [channels unique(ch)];
    end
end

s.trials = trial;
s.conds = unique (cond_no(1:trial));
s.blocks = unique (block_no(1:trial));
s.eog_rate = unique (eog_rate(1:trial));
s.KHz_res = unique (KHz_resolution(1:trial));
s.type = unique (trial_type(1:trial));
s.response = unique (response_type(1:trial));
s.error = unique (response_error(1:trial));
s.epp_ch = unique (channels);