function [h,p,ci,stats] = trialttest (c, cluster, k1, k2, varargin)

%@CORTEX/TRIALTTEST  paired t-test, comparing within-trial epochs
%
%   H = TRIALTTEST(COBJ,CLUSTER,KEYS1,KEYS2)   performs a paired t-test
%   comparing the activity of CLUSTER from the COBJ @cortex object at 2
%   different epochs within the trial. KEYS1 is a @key object (containing
%   any number of keys) that determines the trials to be included as well
%   as the first analysis epoch. KEYS2 is a similar @key object (with the
%   same number of keys as in KEYS1) specifying the cue event, and the
%   start and end ms of the second epoch. Note that any other information
%   contained in KEYS2 is ignored (i.e. trial selection is only done
%   through KEYS1). The function returns 1 if the null hypothesis must be
%   rejected, 0 otherwise.
%
%   H = TRIALTTEST(COBJ,CLUSTER,KEYS1,EPOCH2)   same as above, except that
%   the second epoch is specified by EPOCH2, a 3-element vector indicating
%   the start (ms), end (ms), and cue event code of the second analysis
%   epoch. If the cue event code is omitted, then the second epoch will be
%   locked to the same cue event used for the first epoch (specified by
%   KEYS1).
%
%   H = TRIALTTEST(...,ALPHA)
%   H = TRIALTTEST(...,TAIL)  see >> help ttest
%
%   [H,P] = TRIALTTEST(...)
%   [H,P,CI] = TRIALTTEST(...)
%   [H,P,CI,STATS] = TRIALTTEST(...) see >> help ttest
%
%   Notes: TRIALTTEST is especially useful to test, for example, cell
%   responsivity to a stimulus. This is done by comparing responses before
%   and after stimulus onset. Unlike most functions taking a @key object as
%   argument, this function accepts multi-key objects, since it may make
%   sense to run the test on all meaningful trials at once.
%
%Last modified: 26 Feb 06

error (NOTkey (k1));
if isa (k2, 'key')
	twokeys = true;
	if k1.len ~= k2.len
		error ('the two @key objects must have the same number of keys');
	end
else
	error (NOTnumvector (k2));
	if length(k2) < 2
		error ('EPOCH2 must be a 2- or 3-element vector');
	end
	twokeys = false;
end

di = [];
for k = 1:k1.len
	t1 = trialfiringrate (c, cluster, k1(k));
	if twokeys
		k1(k).cue = k2(k).cue;
		k1(k).sta = k2(k).sta;
		k1(k).fin = k2(k).fin;
	else
		k1(k).sta = k2(1);
		k1(k).fin = k2(2);
		if length(k2)>2, k1(k).cue = k2(3); end
	end
	t2 = trialfiringrate (c, cluster, k1(k));
	di = [di t2-t1];
end

[h,p,ci,stats] = ttest (di, varargin{:});